@@ -73,8 +73,8 @@ class TestSuiteTests(unittest.TestCase):
 
         (ts, tcs) = serialize_and_read(
             TestSuite(
-                'test',
-                [],
+                name='test',
+                test_cases=[],
                 hostname='localhost',
                 id=1,
                 properties=properties,
@@ -100,8 +100,8 @@ class TestSuiteTests(unittest.TestCase):
         timestamp = 1398382805
 
         test_suite = TestSuite(
-                'äöü',
-                [],
+                name='äöü',
+                test_cases=[],
                 hostname='löcalhost',
                 id='äöü',
                 properties=properties,
@@ -131,8 +131,8 @@ class TestSuiteTests(unittest.TestCase):
 
         (ts, tcs) = serialize_and_read(
             TestSuite(
-                decode('äöü', 'utf-8'),
-                [],
+                name=decode('äöü', 'utf-8'),
+                test_cases=[],
                 hostname=decode('löcalhost', 'utf-8'),
                 id=decode('äöü', 'utf-8'),
                 properties=properties,
@@ -198,9 +198,8 @@ class TestSuiteTests(unittest.TestCase):
         verify_test_case(self, suites[1][1][0], {'name': 'Test2'})
 
     def test_attribute_time(self):
-        tss = [TestSuite('suite1',
-               [TestCase('Test1', 'some.class.name', 123.345),
-                TestCase('Test2', 'some2.class.name', 123.345)]),
+        tss = [TestSuite('suite1', [TestCase(name='Test1', classname='some.class.name', elapsed_sec=123.345),
+                                    TestCase(name='Test2', classname='some2.class.name', elapsed_sec=123.345)]),
                TestSuite('suite2', [TestCase('Test2')])]
         suites = serialize_and_read(tss)
 
@@ -212,21 +211,60 @@ class TestSuiteTests(unittest.TestCase):
         # testcase
         self.assertEqual('0', suites[1][0].attributes['time'].value)
 
+    def test_attribute_disable(self):
+        tc = TestCase('Disabled-Test')
+        tc.is_enabled = False
+        tss = [TestSuite('suite1', [tc])]
+        suites = serialize_and_read(tss)
+
+        self.assertEqual('1', suites[0][0].attributes['disabled'].value)
+
+    def test_stderr(self):
+        suites = serialize_and_read(
+            TestSuite(name='test', stderr='I am stderr!',
+                      test_cases=[TestCase(name='Test1')]))[0]
+        self.assertEqual('I am stderr!',
+                         suites[0].getElementsByTagName('system-err')[0].firstChild.data)
+
+    def test_stdout_stderr(self):
+        suites = serialize_and_read(
+            TestSuite(name='test', stdout='I am stdout!',
+                      stderr='I am stderr!',
+                      test_cases=[TestCase(name='Test1')]))[0]
+        self.assertEqual('I am stderr!',
+                         suites[0].getElementsByTagName('system-err')[0].firstChild.data)
+        self.assertEqual('I am stdout!',
+                         suites[0].getElementsByTagName('system-out')[0].firstChild.data)
+
+    def test_no_assertions(self):
+        suites = serialize_and_read(
+            TestSuite(name='test',
+                      test_cases=[TestCase(name='Test1')]))[0]
+        self.assertFalse(suites[0].getElementsByTagName('testcase')[0].hasAttribute('assertions'))
+
+    def test_assertions(self):
+        suites = serialize_and_read(
+            TestSuite(name='test',
+                      test_cases=[TestCase(name='Test1',
+                                           assertions=5)]))[0]
+        self.assertEquals('5',
+                          suites[0].getElementsByTagName('testcase')[0].attributes['assertions'].value)
+
         # @todo: add more tests for the other attributes and properties
 
     def test_to_xml_string(self):
-        test_suites = [TestSuite('suite1', [TestCase('Test1')]),
-                       TestSuite('suite2', [TestCase('Test2')])]
+        test_suites = [TestSuite(name='suite1', test_cases=[TestCase(name='Test1')]),
+                       TestSuite(name='suite2', test_cases=[TestCase(name='Test2')])]
         xml_string = TestSuite.to_xml_string(test_suites)
         if PY2:
             self.assertTrue(isinstance(xml_string, unicode))
         expected_xml_string = textwrap.dedent("""
             <?xml version="1.0" ?>
-            <testsuites errors="0" failures="0" tests="2" time="0.0">
-            \t<testsuite errors="0" failures="0" name="suite1" skipped="0" tests="1" time="0">
+            <testsuites disabled="0" errors="0" failures="0" tests="2" time="0.0">
+            \t<testsuite disabled="0" errors="0" failures="0" name="suite1" skipped="0" tests="1" time="0">
             \t\t<testcase name="Test1"/>
             \t</testsuite>
-            \t<testsuite errors="0" failures="0" name="suite2" skipped="0" tests="1" time="0">
+            \t<testsuite disabled="0" errors="0" failures="0" name="suite2" skipped="0" tests="1" time="0">
             \t\t<testcase name="Test2"/>
             \t</testsuite>
             </testsuites>
@@ -251,23 +289,35 @@ class TestCaseTests(unittest.TestCase):
 
     def test_init_classname(self):
         (ts, tcs) = serialize_and_read(
-            TestSuite('test', [TestCase('Test1', 'some.class.name')]))[0]
+            TestSuite('test',
+                      [TestCase(name='Test1', classname='some.class.name')]))[0]
         verify_test_case(
             self, tcs[0], {'name': 'Test1', 'classname': 'some.class.name'})
 
     def test_init_classname_time(self):
         (ts, tcs) = serialize_and_read(
             TestSuite('test',
-                      [TestCase('Test1', 'some.class.name', 123.345)]))[0]
+                      [TestCase(name='Test1', classname='some.class.name',
+                                elapsed_sec=123.345)]))[0]
         verify_test_case(
             self, tcs[0], {'name': 'Test1', 'classname': 'some.class.name',
                            'time': ("%f" % 123.345)})
 
+    def test_init_classname_time_timestamp(self):
+        (ts, tcs) = serialize_and_read(
+            TestSuite('test',
+                      [TestCase(name='Test1', classname='some.class.name',
+                                elapsed_sec=123.345, timestamp=99999)]))[0]
+        verify_test_case(
+            self, tcs[0], {'name': 'Test1', 'classname': 'some.class.name',
+                           'time': ("%f" % 123.345),
+                           'timestamp': ("%s" % 99999)})
+
     def test_init_stderr(self):
         (ts, tcs) = serialize_and_read(
             TestSuite(
-                'test', [TestCase('Test1', 'some.class.name', 123.345,
-                                  stderr='I am stderr!')]))[0]
+                'test', [TestCase(name='Test1', classname='some.class.name',
+                            elapsed_sec=123.345, stderr='I am stderr!')]))[0]
         verify_test_case(
             self, tcs[0],
             {'name': 'Test1', 'classname': 'some.class.name',
@@ -277,14 +327,21 @@ class TestCaseTests(unittest.TestCase):
         (ts, tcs) = serialize_and_read(
             TestSuite(
                 'test', [TestCase(
-                    'Test1', 'some.class.name', 123.345, 'I am stdout!',
-                    'I am stderr!')]))[0]
+                    name='Test1', classname='some.class.name',
+                    elapsed_sec=123.345, stdout='I am stdout!',
+                    stderr='I am stderr!')]))[0]
         verify_test_case(
             self, tcs[0],
             {'name': 'Test1', 'classname': 'some.class.name',
              'time': ("%f" % 123.345)},
             stdout='I am stdout!', stderr='I am stderr!')
 
+    def test_init_disable(self):
+        tc = TestCase('Disabled-Test')
+        tc.is_enabled = False
+        (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
+        verify_test_case(self, tcs[0], {'name': 'Disabled-Test'})
+
     def test_init_failure_message(self):
         tc = TestCase('Failure-Message')
         tc.add_failure_info("failure message")
@@ -301,13 +358,27 @@ class TestCaseTests(unittest.TestCase):
             self, tcs[0], {'name': 'Failure-Output'},
             failure_output="I failed!")
 
+    def test_init_failure_type(self):
+        tc = TestCase('Failure-Type')
+        tc.add_failure_info(failure_type='com.example.Error')
+        (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
+        verify_test_case(self, tcs[0], {'name': 'Failure-Type'})
+
+        tc.add_failure_info("failure message")
+        (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
+        verify_test_case(
+            self, tcs[0], {'name': 'Failure-Type'},
+            failure_message="failure message",
+            failure_type='com.example.Error')
+
     def test_init_failure(self):
         tc = TestCase('Failure-Message-and-Output')
         tc.add_failure_info("failure message", "I failed!")
         (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
         verify_test_case(
             self, tcs[0], {'name': 'Failure-Message-and-Output'},
-            failure_message="failure message", failure_output="I failed!")
+            failure_message="failure message", failure_output="I failed!",
+            failure_type='failure')
 
     def test_init_error_message(self):
         tc = TestCase('Error-Message')
@@ -324,13 +395,27 @@ class TestCaseTests(unittest.TestCase):
         verify_test_case(
             self, tcs[0], {'name': 'Error-Output'}, error_output="I errored!")
 
+    def test_init_error_type(self):
+        tc = TestCase('Error-Type')
+        tc.add_error_info(error_type='com.example.Error')
+        (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
+        verify_test_case(self, tcs[0], {'name': 'Error-Type'})
+
+        tc.add_error_info("error message")
+        (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
+        verify_test_case(
+            self, tcs[0], {'name': 'Error-Type'},
+            error_message="error message",
+            error_type='com.example.Error')
+
     def test_init_error(self):
         tc = TestCase('Error-Message-and-Output')
         tc.add_error_info("error message", "I errored!")
         (ts, tcs) = serialize_and_read(TestSuite('test', [tc]))[0]
         verify_test_case(
             self, tcs[0], {'name': 'Error-Message-and-Output'},
-            error_message="error message", error_output="I errored!")
+            error_message="error message", error_output="I errored!",
+            error_type="error")
 
     def test_init_skipped_message(self):
         tc = TestCase('Skipped-Message')
@@ -386,9 +471,12 @@ class TestCaseTests(unittest.TestCase):
                 "failure message with illegal unicode char: []"))
 
     def test_init_utf8(self):
-        tc = TestCase('Test äöü', 'some.class.name.äöü', 123.345, 'I am stdöüt!', 'I am stdärr!')
+        tc = TestCase(name='Test äöü', classname='some.class.name.äöü',
+                      elapsed_sec=123.345, stdout='I am stdöüt!',
+                      stderr='I am stdärr!')
         tc.add_skipped_info(message='Skipped äöü', output="I skippäd!")
-        tc.add_error_info(message='Skipped error äöü', output="I skippäd with an error!")
+        tc.add_error_info(message='Skipped error äöü',
+                          output="I skippäd with an error!")
         test_suite = TestSuite('Test UTF-8', [tc])
         (ts, tcs) = serialize_and_read(test_suite, encoding='utf-8')[0]
         verify_test_case(self, tcs[0], {'name': decode('Test äöü', 'utf-8'),
@@ -401,8 +489,11 @@ class TestCaseTests(unittest.TestCase):
                         error_output=decode('I skippäd with an error!', 'utf-8'))
 
     def test_init_unicode(self):
-        tc = TestCase(decode('Test äöü', 'utf-8'), decode('some.class.name.äöü', 'utf-8'), 123.345,
-                      decode('I am stdöüt!', 'utf-8'), decode('I am stdärr!', 'utf-8'))
+        tc = TestCase(name=decode('Test äöü', 'utf-8'),
+                      classname=decode('some.class.name.äöü', 'utf-8'),
+                      elapsed_sec=123.345,
+                      stdout=decode('I am stdöüt!', 'utf-8'),
+                      stderr=decode('I am stdärr!', 'utf-8'))
         tc.add_skipped_info(message=decode('Skipped äöü', 'utf-8'),
                             output=decode('I skippäd!', 'utf-8'))
         tc.add_error_info(message=decode('Skipped error äöü', 'utf-8'),
@@ -413,16 +504,18 @@ class TestCaseTests(unittest.TestCase):
         verify_test_case(self, tcs[0], {'name': decode('Test äöü', 'utf-8'),
                                         'classname': decode('some.class.name.äöü', 'utf-8'),
                                         'time': ("%f" % 123.345)},
-                        stdout=decode('I am stdöüt!', 'utf-8'), stderr=decode('I am stdärr!', 'utf-8'),
-                        skipped_message=decode('Skipped äöü', 'utf-8'),
-                        skipped_output=decode('I skippäd!', 'utf-8'),
-                        error_message=decode('Skipped error äöü', 'utf-8'),
-                        error_output=decode('I skippäd with an error!', 'utf-8'))
+                         stdout=decode('I am stdöüt!', 'utf-8'),
+                         stderr=decode('I am stdärr!', 'utf-8'),
+                         skipped_message=decode('Skipped äöü', 'utf-8'),
+                         skipped_output=decode('I skippäd!', 'utf-8'),
+                         error_message=decode('Skipped error äöü', 'utf-8'),
+                         error_output=decode('I skippäd with an error!', 'utf-8'))
 
 
 def verify_test_case(tc, test_case_element, expected_attributes,
-                     error_message=None, error_output=None,
+                     error_message=None, error_output=None, error_type=None,
                      failure_message=None, failure_output=None,
+                     failure_type=None,
                      skipped_message=None, skipped_output=None,
                      stdout=None, stderr=None):
     for k, v in expected_attributes.items():
@@ -450,6 +543,10 @@ def verify_test_case(tc, test_case_element, expected_attributes,
             tc.assertEqual(
                 error_message, errors[0].attributes['message'].value)
 
+        if error_type and errors:
+            tc.assertEqual(
+                error_type, errors[0].attributes['type'].value)
+
         if error_output:
             tc.assertEqual(
                 error_output, errors[0].firstChild.nodeValue.strip())
@@ -464,6 +561,10 @@ def verify_test_case(tc, test_case_element, expected_attributes,
             tc.assertEqual(
                 failure_message, failures[0].attributes['message'].value)
 
+        if failure_type and failures:
+            tc.assertEqual(
+                failure_type, failures[0].attributes['type'].value)
+
         if failure_output:
             tc.assertEqual(
                 failure_output, failures[0].firstChild.nodeValue.strip())
